﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.IO;

using System.Web; //Add reference to System.Web
using System.Runtime.Serialization.Json; //Add reference to System.Runtime.Serialization

namespace RedCritterConnecter.Samples
{
    //Create a class to contain the response
    public class SendNotificationResponse
    {
        public String ErrorMessage { get; set; }
        public Boolean Submitted { get; set; }
        public String RequestID { get; set; }
        public String APIVersion { get; set; }
        public String ReturnParam1 { get; set; }
        public String ReturnParam2 { get; set; }
        public String ReturnParam3 { get; set; }
        private Int32 ErrorCode { get; set; }
    }

    public class SendNotification
    {

        public delegate void OnSendNotificationResponse(SendNotificationResponse SendNotificationResponse);
        public event OnSendNotificationResponse onSendNotificationResponse;

        public delegate void OnSendNotificationResponseError(SendNotificationResponse SendNotificationResponse);
        public event OnSendNotificationResponseError onSendNotificationResponseError;

        const string CONST_ConnecterBaseURL = "https://www.redcritterconnecter.com/";

        
        public void Execute(String SecretKey, String ExternalAccountID, String ProfileEntity, String Title, String Message)
        {
            try
            {
                //Create url encoded parameters in query string
                String queryString = "secretkey=" + System.Web.HttpUtility.UrlEncode(SecretKey) + "&externalaccountid=" + System.Web.HttpUtility.UrlEncode(ExternalAccountID) + "&profileentity=" + System.Web.HttpUtility.UrlEncode(ProfileEntity) + "&title=" + System.Web.HttpUtility.UrlEncode(Title) + "&message=" + System.Web.HttpUtility.UrlEncode(Message);

                //Create a new instance of a WebClient
                WebClient wc = new System.Net.WebClient();

                //Prevent this request from caching in order to ensure that it is sent to server
                wc.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.NoCacheNoStore);

                //Attach an event handler to receive the response
                wc.DownloadStringCompleted += onSendNotificationResponseReceived;

                //Make the call
                wc.DownloadStringAsync(new Uri(CONST_ConnecterBaseURL + "services/gamificationv1/SendNotification?" + queryString, UriKind.Absolute));
            }
            catch
            {
                //Something went wrong communicating with the server
                if (onSendNotificationResponseError != null)
                {
                    onSendNotificationResponseError(null);
                }
            }

        }

        private void onSendNotificationResponseReceived(object sender, DownloadStringCompletedEventArgs e)
        {
            try
            {
                //Create a JSON serializer
                System.Runtime.Serialization.Json.DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(SendNotificationResponse));

                //Copy the string into a memory stream
                MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(e.Result));

                //Read the stream into an object matching the response's type
                SendNotificationResponse SendNotificationResponse = (SendNotificationResponse)s.ReadObject(ms);

                //Work with the populated response object
                if (SendNotificationResponse.Submitted == true)
                {
                    //Request was accepted, raise the success event
                    onSendNotificationResponse(SendNotificationResponse);
                }
                else
                {
                    //Something went wrong
                    if (onSendNotificationResponseError != null)
                    {
                        //Request failed, raise the error event
                        onSendNotificationResponseError(SendNotificationResponse);
                    }
                }
            }
            catch
            {
                //Something went wrong
                onSendNotificationResponseError(null);
            }
        }
    }
}
